/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <qpixmap.h>
#include <qevent.h>
#include <qtimer.h>

#include "chat.h"
#include "debug.h"
#include "screenshot.h"
#include "chatbutton.h"

#define MODULE_SCREENSHOT_VERSION 0.2.0

ScreenShot* screenShot;

extern "C" int screenshot_init()
{
	kdebugf();
	screenShot = new ScreenShot();
	return 0;
}

extern "C" void screenshot_close()
{
	kdebugf();
	delete screenShot;
}

ScreenShot::ScreenShot(QWidget * parent, const char * name, WFlags f)
 : QWidget(parent, name, Qt::WStyle_Customize|Qt::WStyle_NoBorder)
{
	kdebugf();

	connect(chat_manager, SIGNAL(chatCreated(const UinsList&)), this, SLOT(chatCreated(const UinsList&)));
	connect(chat_manager, SIGNAL(chatDestroying(const UinsList&)), this, SLOT(chatDestroying(const UinsList&)));
	ChatList::ConstIterator it;
	for ( it = chat_manager->chats().begin(); it != chat_manager->chats().end(); it++ )
		handleCreatedChat(*it);

	resize(1, 1);
}

ScreenShot::~ScreenShot()
{
	kdebugf();

	disconnect(chat_manager, SIGNAL(chatCreated(const UinsList&)), this, SLOT(chatCreated(const UinsList&)));
	disconnect(chat_manager, SIGNAL(chatDestroying(const UinsList&)), this, SLOT(chatDestroying(const UinsList&)));
	ChatList::ConstIterator it;
	for ( it = chat_manager->chats().begin(); it != chat_manager->chats().end(); it++ )
		handleDestroyingChat(*it);
}

void ScreenShot::handleCreatedChat(Chat* chat)
{
	kdebugf();

	ChatButton* ss = new ChatButton(chat);
	ss->setPixmap(icons_manager.loadIcon(dataPath("kadu/modules/data/screenshot/icon.png")));
	ss->show();
	QToolTip::add(ss, QT_TRANSLATE_NOOP("@default", "Send ScreenShot"));
	chatButtons[chat] = ss;
	connect(ss, SIGNAL(clicked(Chat*)), this, SLOT(takeShot(Chat*)));
}

void ScreenShot::handleDestroyingChat(Chat* chat)
{
	kdebugf();

	disconnect(chatButtons[chat], SIGNAL(clicked(Chat*)), this, SLOT(takeShot(Chat*)));
	delete chatButtons[chat];
	chatButtons.remove(chat);
}

void ScreenShot::chatCreated(const UinsList& ul)
{
	kdebugf();
	Chat* chat = chat_manager->findChatByUins(ul);
	handleCreatedChat(chat);
}

void ScreenShot::chatDestroying(const UinsList& ul)
{
	kdebugf();
	Chat* chat = chat_manager->findChatByUins(ul);
	handleDestroyingChat(chat);
}

void ScreenShot::mousePressEvent(QMouseEvent* e)
{
	kdebugf();
	startX = e->globalX();
	startY = e->globalY();
}

void ScreenShot::mouseReleaseEvent(QMouseEvent* e)
{
	kdebugf();
	releaseMouse();
	hide();
	int w, x, x2 = e->globalX();
	int h, y, y2 = e->globalY();
	if (x2 > startX)
	{
		x = startX;
		w = x2-startX;
	}
	else
	{
		x = x2;
		w = startX-x2;
	}

	if (y2 > startY)
	{
		y = startY;
		h = y2-startY;
	}
	else
	{
		y = y2;
		h = startY-y2;
	}

	QString path = ggPath("images/shot") +
		QString::number(QDateTime::currentDateTime().toTime_t()) + ".png";

	QPixmap::grabWindow(
			QApplication::desktop()->winId(),
			x, y, w, h
		)
		.save(path, "PNG");

	if (!tempChat)
	{
		printf("ScreenShot module can't determinate Chat window. It should not happen! It's a bug!\n Please report it at Kadu Forum or to module author.\n");
		return;
	}

	if (cleanW > -1)
		painter->drawPixmap(cleanX, cleanY, pixmap, cleanX, cleanY, cleanW, cleanH);

	tempChat->edit->getCursorPosition(&y, &x);
	tempChat->edit->insertAt(QString("[IMAGE ") + path + "]", y, x); // Here we paste the [IMAGE]
	tempChat->edit->moveCursor(QTextEdit::MoveEnd, FALSE);
	tempChat = NULL;

	delete painter;
	painter = NULL;
}

void ScreenShot::takeShot(Chat* chat)
{
	kdebugf();
	tempChat = chat;
	painter = new QPainter(QApplication::desktop()->screen(), TRUE);
	painter->setPen(QColor("grey"));
	pixmap = QPixmap::grabWindow(QApplication::desktop()->winId());
	cleanW = -1;
	cleanH = -1;
	show();
	update();
	QTimer::singleShot(100, this, SLOT(grabMouseSlot()));
}

void ScreenShot::grabMouseSlot()
{
	kdebugf();
	grabMouse();
}

void ScreenShot::mouseMoveEvent(QMouseEvent* e)
{
	kdebugf();
	if (cleanW > -1)
		painter->drawPixmap(cleanX, cleanY, pixmap, cleanX, cleanY, cleanW, cleanH);

	painter->drawRect(startX, startY, e->globalX()-startX, e->globalY()-startY);
	cleanW = e->globalX()-startX;
	cleanH = e->globalY()-startY;
	cleanX = startX;
	cleanY = startY;
	if (cleanW < 0)
	{
		cleanW = -cleanW;
		cleanX = startX-cleanW;
	}

	if (cleanH < 0)
	{
		cleanH = -cleanH;
		cleanY = startY-cleanH;
	}
	cleanW += 2;
	cleanH += 2;
	cleanX--;
	cleanY--;
}

